# Bell 202 Modem monitor. 
# Copyright 2009 Philip C. Pilgrim.
# See end of file for terms of use.

use strict;

use Tk ':eventtypes';
require Tk::Photo;
require Tk::Canvas;
require Tk::Scale;
use Win32;
use Win32::SerialPort qw(:STAT);
use Win32::Registry;
require Win32::API;
use Win32::Clipboard;

require "utf8_heavy.pl";
require "unicore/lib/SpacePer.pl";
require "unicore/To/Lower.pl";
require "unicore/To/Upper.pl";
require "unicore/To/Fold.pl";
require "unicore/lib/Digit.pl";
require "unicore/lib/Word.pl";
require "Encode/Unicode.pm";

$| = 1;

my ($Port, $Stream, @Hist, @Tot, @N, $Balance, $Received, $Scanning);
my @ParamNames = ('#HYST', '#SLICE', '#NOISE');
my @Params = (0x020, 0x025, 0x100);
my @ParamLabels;
my $Registry;
$::HKEY_LOCAL_MACHINE->Open("HARDWARE\\DEVICEMAP\\SERIALCOMM", $Registry) or die "Can't open data: $^E";
my @PortNames = ();

my $Clip = Win32::Clipboard();

my $mw = Tk::MainWindow->new(-title => "Modem Signal Quality Monitor");
$mw->withdraw;
$mw->resizable(0, 0);
$mw->protocol('WM_DELETE_WINDOW' => \&Shutdown);
$mw->Label(
	-textvariable => \$Scanning,
	-width => 40,
	-height => 5,
	-font => 'system 10 bold',
	-background => 'BLACK',
	-foreground => 'GREEN'
)->pack;
$mw->update;
my $dx = int(($mw->screenwidth - $mw->reqwidth)/2);
my $dy = int(($mw->screenheight - $mw->reqheight)/2);
$mw->geometry("+$dx+$dy");
$mw->deiconify;
$mw->update;

&OpenPort;

$mw->withdraw;

$mw = Tk::MainWindow->new(-title => "Modem Signal Quality Monitor");
$mw->withdraw;
$mw->resizable(0, 0);
$mw->protocol('WM_DELETE_WINDOW' => \&Shutdown);
my $imgPlot = $mw->Photo(
	-height => 513,
	-width => 513,
	-palette => '256/256/256'
);

$imgPlot->blank;

my $fraText = $mw->Frame()->pack(-side => 'right');

$fraText -> Label(
	-background => 'BLACK',
	-foreground => 'GREEN',
	-width => 64,
	-height => 48,
	-borderwidth => 2,
	-relief => 'sunken',
	-textvariable => \$Received,
	-justify => 'left',
	-anchor => 'nw'
)->pack(-expand => 1, -fill => 'y', -padx => 5, -pady => 5);


my $canPlot = $mw->Canvas(
	-height => 516,
	-width => 516,
	-background => 'BLACK'
)->pack;

my $fra = $mw->Frame->pack(-pady => 5);

$fra->Button(
	-text => 'COPY',
	-command => \&Copy
)->pack(-side => 'left', -padx => 5);

$fra->Button(
	-text => 'CLEAR',
	-command => \&Clear
)->pack(-side => 'left', -padx => 5);

$canPlot->createImage(4, 4, -image => $imgPlot, -anchor => 'nw');

foreach (0, 64, 128, 192, 256, 320, 384, 448, 512) {
	$canPlot->createLine(4, $_ + 4, 517, $_ + 4, -fill => 'GRAY');
	$canPlot->createLine($_ + 4, 4, $_ + 4, 517, -fill => 'GRAY')
}
$canPlot->createLine(4, 4, 517, 517, -fill => 'WHITE', tags => 'noise');

my $scaBalance = $mw->Scale(
	-orient => 'horizontal',
	-width => 10,
	-sliderlength => 10,
	-length => 128,
	-borderwidth => 0,
	-troughcolor => 'BLACK',
	-from => -160,
	-to => 160,
	-variable => \$Balance
);	

$canPlot->createWindow(389, 256, -window => $scaBalance, -anchor => 's');
$canPlot->createText(387 - 48, 210, -text => '0', -fill => 'GREEN');
$canPlot->createText(387 + 48, 210, -text => '1', -fill => 'RED');
$canPlot->createText(387, 210, -text => '?        ?', -fill => 'WHITE');

foreach my $i (0 .. 2) {
	my $fra = $mw->Frame->pack(-expand => 1, -fill => 'x');
	$fra->Label(
		-text => $ParamNames[$i]
	)->pack(-side => 'left');
	
	$fra->Label(
		-background => 'BLACK',
		-foreground => 'GREEN',
		-borderwidth => 2,
		-relief => 'sunken',
		-width => 10,
		-font => 'systemfixed 9 roman',
		-textvariable => \$ParamLabels[$i]
	)->pack(-side => 'right');
	
	$fra->Scale(
		-orient => 'horizontal',
		-length => 384,
		-width => 15,
		-from => $i == 1 ? -0x400 : 0,
		-to => $i == 2 ? 0x200 : 0x400,
		-sliderlength => 15,
		-troughcolor => 'BLACK',
		-variable => \$Params[$i],
		-showvalue => 0,
		-command => [\&SetParam, $i]
	)->pack(-side => 'right')
}		


&Clear();

$mw->update;
$dx = int(($mw->screenwidth - $mw->reqwidth)/2);
$dy = int(($mw->screenheight - $mw->reqheight)/2);
$mw->geometry("+$dx+$dy");
$mw->deiconify;
my $UpdateID = $mw->after(10, \&Update);
$mw->MainLoop;


sub Update {
	my $nxt;
	$Port->error_msg(1);
	my (undef, $n, undef, $err) = ($Port->status);
	if ($err) {
		$Port->reset_error;
	}
	if ($n) {
		$Stream .= $Port->input;
		while ($Stream =~ m/(\?)(.)|(!)([\dA-F]{3})([\dA-F]{3})([\dA-F]{2})\n/osg) {
			if (defined $1 && $1 eq '?') {
				my $char = $2;
				$Received =~ s/[^\n\r]*[\n\r]// if $char =~ m/[\n\r]/;
				$Received .= $char if $char =~ m/[\n\r\x20-\xff]/;
			} else {
				my $x = hex($4);
				my $y = hex($5);
				my $s = hex($6);
				#print "$1:$2:$3\n";
				$s = ($s + 16) & 0x3f;
				$Hist[$s]++;
				my $i = $s < 16 ? 0 : 1;
				$Tot[$i] += $s - 16;
				$N[$i]++;
				$Balance = ($Tot[0] / $N[0] + $Tot[1] / $N[1]) * 5;
				my $color = abs($s - 16) < 8 ? 'WHITE' : $s >= 16 ? 'RED' : 'GREEN';
				$imgPlot->put($color, -to => 319 + $s * 4, 192, 323 + $s * 4, 192 - $Hist[$s] / 128) if $Hist[$s] < 192 * 128;
				$color = $x & 0x800 && $y & 0x800 ? 'YELLOW' : $x & 0x800 ? 'GREEN' : $y & 0x800 ? 'RED' : 'WHITE';
				$x = ($x >> 0) & 0x1ff;
				$y = 511 - (($y >> 0) & 0x1ff);
				$imgPlot->put($color, -to => $x, $y);
			}
			$nxt = pos($Stream);
		}
		$Stream = substr($Stream, $nxt) if defined $nxt;
	}
	$UpdateID = $mw->after(10, \&Update);
}

sub OpenPort {
	while (1) {		
		my ($port, %coms);
		$Registry->GetValues(\%coms);
		@PortNames = grep {m/^COM\d+$/} (map {$coms{$_}->[2]} keys %coms);
		foreach my $portn (@PortNames) {
			$port->close if $port;
			$Scanning = "Scanning $portn for monitor output.";
			$mw->DoOneEvent(ALL_EVENTS);
			my $portname = "\\\\.\\$portn";
			if (
				$port = new Win32::SerialPort($portname)
				and	$port->baudrate(38400)
				and $port->parity('none')
				and $port->databits(8)
				and $port->stopbits(2)
				and $port->handshake('none')
				and $port->buffers(4096,4096)
				and $port->binary(1)
				and $port->datatype('raw')
				and $port->write_settings
			) {
				Win32::Sleep(1000);
				my $test = $port->input;
				if ($test =~ m/![\dA-F]{8}\n/) {
					$Port = $port;
					return
				}
			}				
		}
	}
}

sub SetParam {
	my $par = shift;
	my $value = $Params[$par] & 0xfff;
	my $param = substr($ParamNames[$par], 1, 1);
	$Port->write(sprintf("!$param%3.3X", $value));
	$ParamLabels[$par] = sprintf('$%3.3X0_0000', $value);
	&Clear();
	$canPlot->coords('noise', 4, 517 - $value, 4 + $value, 517) if $par == 2
}

sub Copy {
	my $clip = '';
	foreach my $i (0 .. 2) {
		$clip .= sprintf('mdm.set(mdm%s, $%3.3x0_0000)', $ParamNames[$i], $Params[$i] & 0xfff) . "\n"
	}
	$Clip->Set($clip)
}

sub Clear {
	$imgPlot->blank;
	undef @Hist;
	@Tot = (0, 0); 
	@N = (0.001, 0.001);
	$Received = "\n" x 47
}


sub Shutdown {
	$mw->afterCancel($UpdateID) if $UpdateID;
	$Port->close if $Port;
	undef $Port;
	exit
}

__END__

+------------------------------------------------------------------------------------------------------------------------------+
                                                   TERMS OF USE: MIT License                                                                                                              
+------------------------------------------------------------------------------------------------------------------------------
Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation     
files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy,    
modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software
is furnished to do so, subject to the following conditions:                                                                   
                                                                                                                              
The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.
                                                                                                                              
THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE          
WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR         
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,   
ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                         
+------------------------------------------------------------------------------------------------------------------------------+
